 /*
 * Project: Device in Jar. Greenhouse.
 * Device:  Main Atmega32
 * File:    One of the bases of the idea. Data storages keep every data interesting for other beings of the system.
 *          Each data storage send events to mark it has been changed. Other subsystems react on that.
 *
 * License: freeware. Impressed by other cool ideas, will be happy if I inspire to new ideas.
 *
 * © 2010 by Maxim Ushakov; www.maxamuxa.com; head@maxamuxa.com
 */
#include "dataStorage.h"

Time::Time( uint8_t hour, uint8_t minute, uint8_t second )
 : m_hour( hour ), m_minute( minute ), m_second( second )
{
}

bool Time::operator >( Time const& secondOperand ) const
{
  //hours bigger
  if( m_hour > secondOperand.m_hour )
    return true;
  else
  {
    //hours smaller
    if( m_hour < secondOperand.m_hour )
      return false;
    //hours equal, go to minutes
    else
    {
      //minutes bigger
      if( m_minute > secondOperand.m_minute )
        return true;
      else
      {
        //minutes smaller
        if( m_minute < secondOperand.m_minute )
          return false;
        //minutes equal, go to seconds
        return m_second > secondOperand.m_second;   
      }
    }
  }
}

ClockData::ClockData()
 : m_time( 0, 0, 0 )
{
 m_dayOfWeek = 0;
}

void ClockData::fireEvent_changed()
{
 throwEvent( event_ClockData_changed );
}

bool ClockData::isWeekday() const
{
 return m_dayOfWeek < 5;
}

void ButtonsData::fireEvent_changed()
{
 throwEvent( event_ButtonsData_changed );
}

void TemperatureData::fireEvent_changed()
{
 throwEvent( event_TemperatureData_changed );
}

bool TemperatureData::isValid( int8_t temperature )
{
 //there is the range of TMP36
 return temperature >= -40 && temperature <= 125;
}

ClimateData::ClimateData()
 : m_nightTemperature( 25 ), m_dayTemperature( 35 ),
   m_dayLength( 12 ), m_wateringInterval( 3 )
{
}

void ClimateData::fireEvent_changed()
{
 throwEvent( event_ClimateData_changed );
}

StateData::StateData()
 : m_dayNow( false ), m_watering( false ), m_badTemperature( false )
{
 m_heaterWorks[ 0 ] = m_heaterWorks[ 1 ] = false;
}

void StateData::fireEvent_changed()
{
 throwEvent( event_StateData_changed );
}

HeapData::HeapData()
 : m_weekdayDawn( 7, 0, 0 ), m_weekendDawn( 10, 0, 0 ),
   m_manualLightInterval( 2, 0, 0 )
{
}

void HeapData::fireEvent_changed()
{
 throwEvent( event_HeapData_changed );
}

//the only objects used everywhere
ClockData clockData;
ButtonsData buttonsData;
TemperatureData temperatureData;
ClimateData climateData;
StateData stateData;
HeapData heapData;