 /*
 * Project: Device in Jar. Greenhouse.
 * Device:  Main Atmega32
 * File:    One of the bases of the idea. Events are sent between everything and build almost all the activyty
 *          of the system.
 *
 * License: freeware. Impressed by other cool ideas, will be happy if I inspire to new ideas.
 *
 * © 2010 by Maxim Ushakov; www.maxamuxa.com; head@maxamuxa.com
 */
#include "events.h"
#include "subSystem.h"

void throwEvent( Events event )
{
 switch( event )
 {
  //timers
  case event_SystemTime_40ms:
   buttonsSubsystem.eventsHandler( event );
   ledSubsystem.eventsHandler( event );
   lcdSubsystem.eventsHandler( event );
   break;
  case event_SystemTime_second:
   clockSubsystem.eventsHandler( event );
   heaterSubsystem.eventsHandler( event );
   break;
  case event_ClockSubsystem_minuteElapsed:
   lcdSubsystem.eventsHandler( event );
   daySubsystem.eventsHandler( event );
   lampSubsystem.eventsHandler( event );
   heaterSubsystem.eventsHandler( event );
   break;
  case event_ClockSubsystem_hourElapsed:
   wateringSubsystem.eventsHandler( event );
   break;

  //buttons for leds
  case event_ButtonsData_changed:
   buttonsSubsystem.eventsHandler( event );
   break;
  case event_ButtonsSubsystem_button0Pressed:
  case event_ButtonsSubsystem_button1Pressed:
  case event_ButtonsSubsystem_button2Pressed:
  case event_ButtonsSubsystem_button3Pressed:
  case event_ButtonsSubsystem_button4Pressed:
  case event_ButtonsSubsystem_button0Released:
  case event_ButtonsSubsystem_button1Released:
  case event_ButtonsSubsystem_button2Released:
  case event_ButtonsSubsystem_button3Released:
  case event_ButtonsSubsystem_button4Released:
  case event_ButtonsSubsystem_userComes:
   ledSubsystem.eventsHandler( event );
   break;

  //buttons for leds & LCD
  case event_ButtonsSubsystem_userLeaves:
   ledSubsystem.eventsHandler( event );
   lcdSubsystem.eventsHandler( event );
   break;

  //buttons for LCD
  case event_ButtonsSubsystem_shiftUp:
  case event_ButtonsSubsystem_shiftDown:
  case event_ButtonsSubsystem_switchMode:
  case event_ButtonsSubsystem_inputUp:
  case event_ButtonsSubsystem_inputDown:
  case event_ButtonsSubsystem_inputRepeatedUp:
  case event_ButtonsSubsystem_inputRepeatedDown:
   lcdSubsystem.eventsHandler( event );
   break;
  case event_ButtonsSubsystem_lightLamps:
  case event_ButtonsSubsystem_putOutLamps:
    lampSubsystem.eventsHandler( event );
    break;

  //buttons for the watering subsystem
  case event_ButtonsSubsystem_watering:
   wateringSubsystem.eventsHandler( event );
   break;

  //changes happened to data storages
  case event_ClockData_changed:
  case event_ClimateData_changed:
  case event_HeapData_changed:
   lcdSubsystem.eventsHandler( event );
   daySubsystem.eventsHandler( event );
   break;
  case event_StateData_changed:
   lampSubsystem.eventsHandler( event );
   lcdSubsystem.eventsHandler( event );
   break;
  case event_TemperatureData_changed:
   lcdSubsystem.eventsHandler( event );
   break;
 }
}