 /*
 * Project: Device in Jar. Greenhouse.
 * Device:  Main Atmega32
 * File:    main()
 *
 * License: freeware. Impressed by other cool ideas, will be happy if I inspire to new ideas.
 *
 * © 2010 by Maxim Ushakov; www.maxamuxa.com; head@maxamuxa.com
 */
#include "MPUSettings.h"

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <avr/interrupt.h>

//custom libraries
#include "lcd.h"
#include "twi.h"
#include "timer.h"
#include "adc.h"
#include "subSystem.h"

TWI Twi;

//the issue about power consumption: set all unused pins as inputs with internal pull-up resistors
void decreaseConsumption()
{
 DDRB &= ~( _BV( PIN6 ) | _BV( PIN7 ) );
 PORTB |= _BV( PIN6 ) | _BV( PIN7 );
 
 DDRC &= ~( _BV( PIN2 ) | _BV( PIN7 ) );
 PORTC |= _BV( PIN2 ) | _BV( PIN7 );
 
 DDRD = 0x00;
 PORTD = 0xFF;
}

int main()
{
 lcd_init( LCD_DISP_ON );
 Twi.enable();
 Timer::initialize();
 ButtonsAndADC::initialize();
 lcdSubsystem.initialize();

 //enable interrupts
 sei();

 decreaseConsumption();

 //the main routine - great loop doing nothing
 while( 1 );

 return 0;
}

//Required when pure virtual classes are used.
//They call that nowhere.
extern "C" void __cxa_pure_virtual()
{
 while (1);
}