 /*
 * Project: Device in Jar. Greenhouse.
 * Device:  Main Atmega32
 * File:    Timer generating event cought by the special subsystem (40ms events)
 *
 * License: freeware. Impressed by other cool ideas, will be happy if I inspire to new ideas.
 *
 * © 2010 by Maxim Ushakov; www.maxamuxa.com; head@maxamuxa.com
 */
#include "timer.h"

#include <avr/io.h>
#include <avr/interrupt.h>

#include "events.h"

void Timer::initialize()
{
 //init the timer; settings:
 // -Timer1 is in use
 // -The core prescaler is 1/1024 (so, 15625Hz)
 // -Counter is set to 625 (so, 25Hz)
 // -The CTC mode
 TCCR1A = 0;
 TCCR1B = 0x0D;
 OCR1A = 0x0271;
 TIMSK = 0x10;
}

void adcReceiverFor40ms();

SIGNAL( TIMER1_COMPA_vect )
{
 //events about 1/25 second
 throwEvent( event_SystemTime_40ms ); 
 adcReceiverFor40ms();
 
 static uint8_t oneTwentyFifth;
 if( oneTwentyFifth++ == 24 )
 {
  throwEvent( event_SystemTime_second );
  oneTwentyFifth = 0;
 }
}