 /*
 * Project: Device in Jar. Greenhouse.
 * Device:  Main Atmega32
 * File:    TWI for the master Atmega32 (transmit/receive 16-bit words)
 *
 * License: freeware. Impressed by other cool ideas, will be happy if I inspire to new ideas.
 *
 * © 2010 by Maxim Ushakov; www.maxamuxa.com; head@maxamuxa.com
 */
/*
  This class is mentioned to support TWI for the master controller.
  It should:
  1. detect timeouts and treat them as terminated sessions
  2. return to the initial state if a session is terminated
  3. understand if other side (slave) has terminated a session

  Hardware configuration:
  MPU: Atmega32
  Clock: 1MHz/16MHz (when the resonator is on)
  Own address: 1
  Address of the slave: 2

  Other notes:
  1. Zero address is used to point real addresses when a data is requested.
     So, if its value is requested, the last requested address is returned.
*/
#include <util/twi.h>

class TWI
{
public:

  enum Result
  {
    DataReceived, //something is received properly
    DataSent, //something is sent properly
    Nothing, //nothing bad
    Timeout, //timeout; it seems that the other side is broken
    Terminated, //the session is terminated by the other side
    Error //some error
  };

  //Enable & disable TWI.
  void enable();
  void disable();

  //Send something.
  //Addresses should be undenstandable by the slave.
  Result sendData( uint8_t address, uint16_t data );

  //Try to ask for something.
  //Addresses should be understandable by the slave.
  Result receiveData( uint8_t address, uint16_t& data );

private:
  //wait until TWINT is set (some operation is finished)
  //or a timeout occurs
  Result waitForTWINT();

  //I hope this method can drop the state from any error
  void dropState();
  
  //this field is used to determine timeouts;
  //should be set to zero if a new operation is starting
  uint16_t m_loopsCounter;
};
